@@ -4,6 +4,8 @@
 # Original version by Paul Legato (plegato@nks.net), 4/20/06.
 #
 # Copyright (C) 2006 Networked Knowledge Systems, Inc.
+# Copyright (C) 2008 Olly Betts
+# Copyright (C) 2010 Richard Boulton
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License as
@@ -122,15 +124,16 @@ module Xapian
 
   # Ruby wrapper for Xapian::ValueIterator
   class Xapian::Value
-    attr_accessor :value, :valueno
+    attr_accessor :value, :valueno, :docid
     
-    def initialize(value, valueno)
+    def initialize(value, valueno, docid)
       @value = value
       @valueno = valueno
+      @docid = docid
     end # initialize
 
     def ==(other)
-      return other.is_a?(Xapian::Value) && other.value == @value && other.valueno == @valueno
+      return other.is_a?(Xapian::Value) && other.value == @value && other.valueno == @valueno && other.docid == @docid
     end
   end # Xapian::Value
 
@@ -148,7 +151,7 @@ module Xapian
 
     def values
       Xapian._safelyIterate(self._dangerous_values_begin(), self._dangerous_values_end()) { |item|
-        Xapian::Value.new(item.value, item.valueno)
+        Xapian::Value.new(item.value, item.valueno, 0)
       }
     end # terms
 
@@ -211,8 +214,8 @@ module Xapian
       Xapian._safelyIterate(self._begin(), 
                             self._end()) { |item|
 	# note: in the ExpandTerm wrapper, we implicitly rename
-	# ESetIterator#termname() (defined in xapian.i) to ExpandTerm#term()
-        Xapian::ExpandTerm.new(item.termname, item.weight)
+	# ESetIterator#term() (defined in xapian.i) to ExpandTerm#term()
+        Xapian::ExpandTerm.new(item.term, item.weight)
       }
 
     end # terms
@@ -275,6 +278,38 @@ module Xapian
         item.termpos
       }
     end # positionlist
+
+    # Returns an Array of Xapian::Value objects for the given slot in the
+    # database.
+    def valuestream(slot)
+      Xapian._safelyIterate(self._dangerous_valuestream_begin(slot),
+                            self._dangerous_valuestream_end(slot)) { |item|
+        Xapian::Value.new(item.value, slot, item.docid)
+      }
+    end # positionlist
+  end # Xapian::Database
+
+  # Refer to the
+  # {Xapian::ValueCountMatchSpy C++ API documentation}[http://xapian.org/docs/apidoc/html/classXapian_1_1ValueCountMatchSpy.html]
+  # for methods not specific to Ruby.
+  #--
+  # Wrap some dangerous iterators.
+  class Xapian::ValueCountMatchSpy
+    # Returns an Array of all the values seen, in alphabetical order
+    def values()
+      Xapian._safelyIterate(self._dangerous_values_begin(),
+                            self._dangerous_values_end()) { |item|
+        Xapian::Term.new(item.term, 0, item.termfreq)
+      }
+    end # allterms
+
+    # Returns an Array of the top values seen, by frequency
+    def top_values(maxvalues)
+      Xapian._safelyIterate(self._dangerous_top_values_begin(maxvalues),
+                            self._dangerous_top_values_end(maxvalues)) { |item|
+        Xapian::Term.new(item.term, 0, item.termfreq)
+      }
+    end # allterms
   end # Xapian::Database
 
 end # Xapian module
